import { noop } from '../fns.ts';
import { SCROLL_LOG_FILE_PREFIX, SCROLL_LOG_FILE_SUFFIX } from '../config.ts';
import { getRuntime } from './helpers.ts';

/**
 * The label for type/severity of the log entry
 */
export enum LogLevel {
	Debug = 'Debug',
	Info = 'Info',
	Notice = 'Notice',
	Warning = 'Warning',
	Error = 'Error',
}

/**
 * Basic logging
 *
 * @param s The string or data to display first
 * @param level The log severity
 * @param data Any additional data to display with the log entry
 */
export function log(
	s: unknown,
	level: LogLevel = LogLevel.Notice,
	data?: any,
): void {
	getRuntime().then(({ file }) => {
		const rawS = JSON.stringify(s, null, 2);
		const rawData = JSON.stringify(data, null, 2);
		const output = (typeof data !== 'undefined')
			? `${rawS}\n${rawData}\n\n`
			: `${rawS}\n`;

		const outputFile =
			`${SCROLL_LOG_FILE_PREFIX}-${level.toLowerCase()}${SCROLL_LOG_FILE_SUFFIX}`;
		file.appendFile(outputFile, output).then(noop);
	});
}

/**
 * Make a log entry of `LogLevel.Debug` severity
 */
export const logDebug = (s: unknown, data?: any) =>
	log(s, LogLevel.Debug, data);

/**
 * Make a log entry of `LogLevel.Info` severity
 */
export const logInfo = (s: unknown, data?: any) => log(s, LogLevel.Info, data);

/**
 * Make a log entry of `LogLevel.Notice` severity
 */
export const logNotice = (s: unknown, data?: any) =>
	log(s, LogLevel.Notice, data);

/**
 * Make a log entry of `LogLevel.Warning` severity
 */
export const logWarning = (s: unknown, data?: any) =>
	log(s, LogLevel.Warning, data);

/**
 * Make a log entry of `LogLevel.Error` severity
 */
export const logError = (s: unknown, data?: any) =>
	log(s, LogLevel.Warning, data);
